package de.hdm_stuttgart.mi.sd1.task1;

import java.util.Arrays;

/**
 * Array helper methods.
 */
public class HelperArray {

  /**
   * <p>Filter an array of strings for creating a new array containing only those
   * non-<code>null</code> values following a given reference string. The arguments array is being untouched.</p>
   *
   * <p>Examples starting from the series of strings
   * <code>{"brown", "red", null, "yellow", "green"}</code>:</p>
   *
   <table class="goikTableDefaults">
   *   <caption>Filtering results with respect to <code>{"brown", "red", null, "yellow", "green"}</code></caption>
   *   <tr>
   *     <th>Reference</th>
   *     <th colspan="2">Result</th>
   *   </tr>
   *
   *   <tr>
   *       <td><code>"magenta"</code></td>
   *       <td><code>{"red", "yellow"}</code></td>
   *       <td>Both <code>"brown"</code> and <code>"green"</code> precede "magenta" alphabetically and thus get
   *       evicted. The <code>null</code> value is being ignored.</td>
   *   </tr>
   *
   *   <tr>
   *       <td><code>"red"</code></td>
   *       <td><code>{"yellow"}</code></td>
   *       <td>All values except <code>{"yellow"}</code> precede <code>"red"</code>.</td>
   *   </tr>
   *
   * </table>
   *
   * <section class="implementationHints">
   *
   *     <h4>Implementation hint:</h4>
   *
   *     <ul>
   *         <li>See {@link String#compareTo(String)}.</li>
   *     </ul>
   *
   * </section>
   *
   * @param values {@link String} or <code>null</code> containing {@link String} and/or <code>null</code> values.
   * @param reference A reference string defining a filter.
   * @return A possibly empty array containing only those non-<code>null</code> values following the
   * reference string in alphabetical order. If either argument is <code>null</code> the result will be an empty array.
   */
  static public String[] getAlphabeticalFollowers(final String[] values, final String reference) {

    if (null == values || null == reference) {                 // Filter special cases returning an empty
      return new String[] {};                                  // result array.
    } else {
      final String[] resultBuffer = new String[values.length]; // Container for collecting results.
      int numFollower = 0;                                     // No follower found yet

      for (final String s: values) {                           // Search all non-null input[] values for alphabetical
        if (null != s && 0 < s.compareTo(reference)) {         // followers with respect to reference and append
          resultBuffer[numFollower++] = s;                     // them to resultBuffer[] each time incrementing
        }                                                      // numFollower.
      }
      return Arrays.copyOf(  // Create result array containing only
          resultBuffer,      // alphabetical followers
          numFollower);

      // Much better: Using Java streams, subject in «Software Development 2»

      //      return Arrays.stream(values)
      //          .filter(Objects::nonNull)
      //          .filter(s -> 0 < s.compareTo(reference))
      //          .toArray(String[]::new);
    }
  }

  /**
   * <p>An element <strong>e</strong> of an integer array is called a leader if either of the following two
   * statements hold:</p>
   *
   * <ul>
   *   <li>
   *     <p><strong>e</strong> is larger than or equal to all of its right neighbours.</p>
   *   </li>
   *   <li>
   *     <p><strong>e</strong> has got no right neighbours i.e. is itself the last array element.</p>
   *     <p>Consequence: An array's rightmost element is always a leader.</p>
   *   </li>
   * </ul>
   *
   * <p>Example: The array <strong>[1, 4, -2, 0, 3]</strong> contains two leaders [4, 3] at array index positions
   * 1 and 4 respectively.</p>
   *
   * @param values A non-empty sequence of values.
   * @return An array containing all leader values including possible duplicates preserving their order of appearance
   * within the original array.
   */
  static public int[] getLeaders(final int[] values) {

    final int[] candidates = new int[values.length];             // Maximum possible length.
    int currentLeaderIndex = values.length - 1;                  // Rightmost element is
    candidates[currentLeaderIndex] = values[currentLeaderIndex]; // always leader.

    for (int i = values.length - 2; 0 <= i; i--) {               // Search values[] for further
      if (candidates[currentLeaderIndex] <= values[i]){          // leader elements and copy those
        candidates[--currentLeaderIndex] = values[i];            // into candidates[].
      }
    }
    return Arrays.copyOfRange(     // Create result array from «lower» part
        candidates,                // of candidates[] containing at least one leader.
        currentLeaderIndex,
        values.length);
  }
}
