package de.hdm_stuttgart.mi.sd1.task2;

import java.util.Arrays;

/**
 * Square Matrix representations. See the {@link de.hdm_stuttgart.mi.sd1.task2.demo.DemoDriver} example.
 */
public class Matrix {

    private final int[][] values;
    private final int size;

    /**
     * Initializing a square matrix.
     *
     * The two dimensional values array must define a square matrix having equal row and column counts like e.g.:
     *
     * <pre>m = new Matrix(new int[][]
     *             {
     *                     {1, 2},
     *                     {3, 4}
     *             })</pre>
     *
     * <code>m</code> will then represent a 2 x 2 matrix:
     *
     * \[
     * \begin{bmatrix}
     * 1 &amp; 2 \\
     * 3 &amp; 4 \\
     * \end{bmatrix}
     * \]
     *
     * On contrary the following argument will cause an {@link ArithmeticException}:
     *
     * <pre>m = new Matrix(new int[][]
     *             {
     *                     {1, 2, 3},
     *                     {4, 5}, // does not suit size 3 x 3 square matrix
     *                     {6, 7, 8}
     *             })</pre>
     *
     * @param values A two dimensional array holding the intended matrix values in a row/column fashion.
     * @throws ArithmeticException Raised if the input array does not represent a (n x n) square matrix.
     */
    public Matrix(final int[][] values) throws ArithmeticException {
        this.values = values;
        size = values.length;
        for (final int[] row: values){
            if (row.length != size) {
                throw new ArithmeticException("Array does not define a square matrix");
            }
        }
    }

    /**
     * The usual component wise matrix addition.
     *
     * Example:
     *
     * \[
     * \begin{bmatrix}
     * 1 &amp; 2 &amp; 3 \\
     * 4 &amp; 5 &amp; 6 \\
     * 7 &amp; 8 &amp; 9 \\
     * \end{bmatrix}
     * +
     * \begin{bmatrix}
     * 1 &amp; 1 &amp; 1 \\
     * 1 &amp; 1 &amp; 1 \\
     * 1 &amp; 1 &amp; 1 \\
     * \end{bmatrix}
     * =
     * \begin{bmatrix}
     * 2 &amp; 3 &amp; 4 \\
     * 5 &amp; 6 &amp; 7 \\
     * 8 &amp; 9 &amp; 10 \\
     * \end{bmatrix}
     * \]
     *
     * @param other Another matrix to be added to the current instance.
     * @return The component wise sum of both matrix instances
     * @throws ArithmeticException In case the size of both matrix instances differs.
     *
     */
    public Matrix add(final Matrix other) throws ArithmeticException {

        if (size != other.size) {
            throw new ArithmeticException("Matrix sizes differ: " + size + " != " + other.size);
        }

        final int[][] result = new int[size][];

        for (int row = 0; row < size; row++) {
            result[row] = Arrays.copyOf (values[row], size);
            for (int col = 0; col < size; col++) {
                result[row][col] += other.values[row][col];
            }
        }
        return new Matrix(result);
    }

    /**
     * Access matrix element by row and column.
     *
     * We consider an example matrix:
     *
     * \[
     * \begin{bmatrix}
     * 1 &amp; 2 &amp; 3 \\
     * 4 &amp; 5 &amp; 6 \\
     * 7 &amp; 8 &amp; 9 \\
     * \end{bmatrix}
     * \]
     *
     * The above matrix has got size 3 x 3. Rows and column index values range from 0 to 2. So when <code>m</code>
     * represents the above matrix executing <code>m.get(1, 2)</code> will select the second row and the third column
     * thus returning 6.
     *
     * @param row Row index ranging from 0 to the matrix size-1
     * @param col Column index ranging from 0 to the matrix size-1
     * @return The matrix value pointed to by row and column specification.
     */
    public int get(int row, int col) {
        return values[row][col];
    }

    /**
     * Two square matrices are equal if:
     *
     * <ol>
     *     <li>The sizes of both instances are equal</li>
     *     <li>All corresponding values (same row and column) are equal</li>
     * </ol>
     *
     * @param obj Instance to be compared.
     * @return true if object is a matrix being equal to the current instance.
     */
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Matrix) {
            final Matrix other = (Matrix) obj;
            if (size == other.size) {
                for (int row = 0; row < size; row++) {
                    if (! Arrays.equals(values[row], other.values[row])) {
                        return false;
                    }
                }
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
}
