package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Providing zeroes (German: "Nullstellen") of quadratic polynomials.</p>
 *
 * <p>A quadratic polynomial \( p(x)  = a x² + b x + c \) is being defined by its three coefficients
 * \(a\), \(b\) and \(c\). This class limits coefficients \(a\), \(b\) and \(c\) to type <code>int</code>. A general
 * solution is being provided by:</p>
 *
 * <p>\[ x_{1,2} = {{-b \pm \sqrt{b^2 - 4 ac}} \over {2a}} \]</p>
 *
 * <p>Depending on \( b^2 - 4 ac \) being positive, zero or negative we have either two, one or no real solutions.</p>
 *
 * <p>The following sample illustrates zeroes calculation depending on given values of \(a\), \(b\) and \(c\):</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Sample code illustrating zero values calculation</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="language-java"> final QuadratPolynom poly =         // Representing
 *     new QuadratPolynom(4, -3, -10); // p(x) = 4x² - 3x - 10
 *
 * double[] zeroes = poly.getZeroes();
 *
 * System.out.println("Found " + zeroes.length + " zeroes:");
 *
 * System.out.println("x_1 = " + zeroes[0]);
 * System.out.println("x_2 = " + zeroes[1]);</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight"> Found 2 zeroes:
 * x_1 = -1.25
 * x_2 = 2.0</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 * <p>We allow for re-setting a polynomial's coefficients. Continuing from the above example we have:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Changing coefficients</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> ...
 * poly.setA(1);  // Representing
 * poly.setB(-8); // p(x) = x² -8x + 16
 * poly.setC(16);
 *
 * zeroes = poly.getZeroes();
 * System.out.println("Found " + zeroes.length + " zero:");
 *
 * System.out.println("x_1 = " + zeroes[0]);</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight"> ...
 * Found 1 zero:
 * x_1 = 4.0</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 * <p>Some polynomials do not have any (real valued) solution:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Changing coefficients again</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> ...
 * poly.setA(1);  // Representing
 * poly.setB(0);  // p(x) = x² + 1
 * poly.setC(1);  // (No zero solution)
 *
 * zeroes = poly.getZeroes();
 * System.out.println("Found " + zeroes.length + " zero");</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight"> ...
 * Found 0 zero</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 * <p>Finally setting \(a=0\) in \(ax²\) leaves us with just a linear polynomial \(bx + c\). To
 * avoid unpredictable results the attempt shall result in throwing an
 * <a href="https://freedocs.mi.hdm-stuttgart.de/doc/openjdk-17-doc/api/java.base/java/lang/ArithmeticException.html"
 *  ><code>ArithmeticException</code></a>:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Dealing with 0 value at square coefficient.</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> ...
 * poly.setA(0);  // Trying to represent p(x) = 1</code></pre>
 *             </td>
 *             <td>
 *                 <pre><code class="nohighlight"> ...
 * Exception in thread "main" java.lang.ArithmeticException:
 *     Square coefficient must not be zero</code></pre>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 * <p>Likewise a <code>new QuadratPolynom(0, ..., ...)</code> constructor call setting \(a=0\) shall raise an
 * <a href="https://freedocs.mi.hdm-stuttgart.de/doc/openjdk-17-doc/api/java.base/java/lang/ArithmeticException.html">
 * <code>ArithmeticException</code></a> as well.</p>
 *
 * <section class="implementationHints">
 *    <h2 class="implementationHints">Hint:</h2>
 *
 * <p>This class is yet unimplemented. The above code snippets provide a clue to an implementation
 *    satisfying the corresponding unit tests.</p>
 *
 * </section>
 */
public class QuadratPolynom {
    // TODO: Implement me
    private QuadratPolynom(){/* Ignore me: My sole purpose is suppressing default constructor javadoc generation */}
}
